/*
 * Wide Band RF Frequency Generator
 * Kerry Wong
 * http://www.kerrywong.com
 */

//instruction format
//CMD  ADDR Data (16 bits)
//xxxx XXXX XXXXXXXX XXXXXXXX
const byte CMD_NONE = 0x0;
const byte CMD_RSTR_EEMEM_RDAC = 0x1;
const byte CMD_STR_RDAC_EEMEM = 0x2;
const byte CMD_STR_RDAC_USR_DAT_EEMEM = 0x3;
const byte CMD_DEC_6DB = 0x4;
const byte CMD_DEC_ALL_6DB = 0x5;
const byte CMD_DEC_ONE_STEP = 0x6;
const byte CMD_DEC_ALL_ONE_STEP = 0x7;
const byte CMD_RSET_RDAC = 0x8;
const byte CMD_READ_EEMEM_FROM_SDO = 0x9;
const byte CMD_READ_RDAC_WIPER_FROM_SDO = 0xA;
const byte CMD_WR_RDAC = 0xB;
const byte CMD_INC_6DB = 0xC;
const byte CMD_INC_ALL_6DB = 0xD;
const byte CMD_INC_ONE_STEP = 0xE;
const byte CMD_INC_ALL_ONE_STEP = 0xF;

//SDO, RDY need 2.2k pull-up resistors
#define DATAOUT 11// SDI
#define DATAIN  12// SDO
#define SPICLOCK  13// SCK
#define CS 10 
#define RELAY 9
#define FREQ_MULT 8

//LTC6905
const float MIN_FREQ = 17.225;
const float MAX_FREQ = 170;

const float FREQ_MIN_X4 = 17.23;
const float FREQ_MAX_X4 = 42.46;
const float FREQ_MIN_X2 = 42.48;
const float FREQ_MAX_X2 = 84.92;
const float FREQ_MIN_X1 = 84.96;
const float FREQ_MAX_X1 = 169.84;

const unsigned int WIPER_MAX = 827;
const unsigned int WIPER_MIN = 410;

const int BTN_FREQ_DOWN = 2; 
const int BTN_FREQ_UP = 3;

int currentMode = 0; //0 - X4, 1 - X2, 2 - X1
unsigned int wiper = WIPER_MIN;

byte spiTransfer(volatile byte data) {
  SPDR = data;

  while (!(SPSR & _BV(SPIF)));

  return SPDR;
}

void initSys()
{
  pinMode(BTN_FREQ_DOWN, INPUT);
  pinMode(BTN_FREQ_UP, INPUT);
  digitalWrite(BTN_FREQ_DOWN, HIGH);
  digitalWrite(BTN_FREQ_UP, HIGH);

  pinMode(DATAOUT, OUTPUT);
  pinMode(DATAIN, INPUT);
  pinMode(SPICLOCK, OUTPUT);
  pinMode(CS, OUTPUT);

  pinMode(RELAY, OUTPUT);
  pinMode(FREQ_MULT, OUTPUT);

  digitalWrite(RELAY, LOW);
  digitalWrite(FREQ_MULT, LOW);

  digitalWrite(CS, HIGH);
  SPCR = _BV(SPE) | _BV(MSTR) | _BV(CPOL) | _BV(CPHA);
}

void setPotValue(byte channel, unsigned int v)
{
  byte command_byte = (CMD_WR_RDAC << 4) | channel; 
  byte data_byte1 = (v >> 8) & 0x3;
  byte data_byte2 = v & 0xff;

  digitalWrite(CS, LOW);
  spiTransfer(command_byte);
  spiTransfer(data_byte1);
  spiTransfer(data_byte2);
  digitalWrite(CS, HIGH);   
}

void setMinFrequency()
{
  //set mode x4
  currentMode = 0;
  digitalWrite(RELAY, HIGH);
  digitalWrite(FREQ_MULT, HIGH); 
  wiper = 1023;
  setPotValue(0, wiper);

}

void setMaxFrequency()
{
  //set mode x1
  currentMode = 2;
  digitalWrite(RELAY, HIGH);
  digitalWrite(FREQ_MULT, LOW); 
  wiper = WIPER_MIN;
  setPotValue(0, wiper);

}

void generateFrequency(float f)
{
  float r_set = 10;

  if (f > MAX_FREQ) {
    f = MAX_FREQ;
  } 
  else if (f < MIN_FREQ) {
    f = MIN_FREQ;
  }

  if (f > FREQ_MIN_X4 && f <= FREQ_MAX_X4)
  {
    //set mode x4
    currentMode = 0;
    r_set = 1685.0 /(4.0 * f - 1.5);
    digitalWrite(RELAY, HIGH);
    digitalWrite(FREQ_MULT, HIGH);    
  } 
  else if (f > FREQ_MIN_X2 && f <= FREQ_MAX_X2)
  {
    //set mode x2
    currentMode = 1;
    r_set = 1685.0 /(2.0 * f - 1.5);
    digitalWrite(RELAY, LOW);
    digitalWrite(FREQ_MULT, LOW);    
  } 
  else if (f > FREQ_MIN_X1 && f <= FREQ_MAX_X1)
  {
    //set mode x1
    currentMode = 2;
    r_set = 1685.0 /(1.0 * f - 1.5); 
    digitalWrite(RELAY, HIGH);
    digitalWrite(FREQ_MULT, LOW);    
  }  

  wiper = (unsigned int) (r_set * 1024.0 / 25.0);

  setPotValue(0, wiper);
}

void setup()
{
  Serial.begin(9600);
  initSys();
  generateFrequency(60.0);
}

float freq = 0;
int idx = 0;
char c[20];

void outputCurrentFrequency()
{
  if (currentMode == 0) //X4
  {
    digitalWrite(RELAY, HIGH);
    digitalWrite(FREQ_MULT, HIGH);       
    freq = (168.5 * 10.0 / (25.0 * wiper / 1024.0) + 1.5) / 4.0;
  } 
  else if (currentMode == 1) //X2
  {
    digitalWrite(RELAY, LOW);
    digitalWrite(FREQ_MULT, LOW); 
    freq = (168.5 * 10.0 / (25.0 * wiper / 1024.0) + 1.5) / 2.0;      
  } 
  else if (currentMode == 2) //X1
  {
    digitalWrite(RELAY, HIGH);
    digitalWrite(FREQ_MULT, LOW);   
    freq = (168.5 * 10.0 / (25.0 * wiper / 1024.0) + 1.5);
  }        

  Serial.print("Frequency = ");
  Serial.print(freq);
  Serial.print(" Mode = ");  
  Serial.print(currentMode);
  Serial.print(" Wiper = ");
  Serial.println(wiper);

}

void loop()
{
  Serial.println("Enter frequency (e.g. 123.45 which means 123.45 Mhz)");
  while (Serial.available() <= 0)
  {   
    if (digitalRead(BTN_FREQ_UP) == LOW) {
      delay(10);
      if (digitalRead(BTN_FREQ_UP) == LOW) {
        wiper--;

        if (wiper < WIPER_MIN) {
          if (currentMode == 0) //X4
          {
            wiper = WIPER_MAX;
            currentMode = 1;
          } 
          else if (currentMode == 1) //X2
          {
            wiper = WIPER_MAX;
            currentMode = 2;
          } 
          else if (currentMode == 2) //X1
          {
            //already at the highest frequency
            wiper = WIPER_MIN;
            currentMode = 2;
          }
        }
        outputCurrentFrequency();
        setPotValue(0, wiper);
        delay(50);
      }
    }

    if (digitalRead(BTN_FREQ_DOWN) == LOW) {
      delay(10);
      if (digitalRead(BTN_FREQ_DOWN) == LOW) {
        wiper++;

        if (wiper > WIPER_MAX) {
          if (currentMode == 0) //X4
          {
            if (wiper >1023)
              wiper = 1023;
            currentMode = 0;          
          } 
          else if (currentMode == 1) //X2
          {
            wiper = WIPER_MIN;
            currentMode = 0;
          } 
          else if (currentMode == 2) //X1
          {
            wiper = WIPER_MIN;
            currentMode = 1;
          }             
        }    
        outputCurrentFrequency();
        setPotValue(0, wiper); 
        delay(50);  
      }      
    }
  }

  idx = 0;
  while (Serial.available() > 0) {
    c[idx] = Serial.read();
    idx++;
    delay(10);
  }

  c[idx] = '\0';
  freq = atof(c);

  generateFrequency(freq);
  Serial.print("Frequency = ");
  Serial.print(freq);
  Serial.println(" Mhz");   
}













